/****************************************************************************
	$Header: /Dev/Tin3/InstallDir/Tix/Tix.h 15    16-08-06 17:22 Ron $

	Copyright (c) 1990-2005 Tarma Software Research. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	2002-10-23 11:29

	Declaration of the Tarma Installer Extension (TIX) interface.

	NOTE: For best results, view with tab stops set at every 4 positions.

	THE CONTENTS OF THIS DOCUMENT ARE CONFIDENTIAL AND PROPRIETARY TO
	TARMA SOFTWARE RESEARCH PTY LTD, AND MAY NOT BE REPRODUCED, PUBLISHED,
	OR DISCLOSED TO OTHERS WITHOUT COMPANY AUTHORISATION.

	Use of this document is allowed to registered users of Tarma Installer
	and solely for the purpose of creating Tarma Installer Extension DLLs.
****************************************************************************/

#ifndef __TIX_H
#define __TIX_H

#pragma once

#ifndef __TIXDEFS_H
#include "TixDefs.h"	// Base declarations
#endif

#pragma pack(push,4)	// Use 4-byte alignment throughout

#ifdef __cplusplus
extern "C" {
#endif

//===========================================================================
// Standard entry point declaration
//
// This entry point should be provided by clients in their extension DLL.
// Its name depends on the compilation model:
//
//	ANSI/MBCS			Unicode
//	----------------	----------------
//	TixHandlerA			TixHandlerW			Name in source code
//	_TixHandlerA@12		_TixHandlerW@12		Exported name
//===========================================================================

// Forward declarations

typedef struct cTixMessage cTixMessage;		// This file
typedef struct iTixRuntimeA iTixRuntimeA;	// See TixRuntime.h
typedef struct iTixRuntimeW iTixRuntimeW;	// See TixRuntime.h

#ifdef _UNICODE
#define iTixRuntime iTixRuntimeW
#else
#define iTixRuntime iTixRuntimeA
#endif

// Prototypes of required handler function

_TIXENTRY BOOL TIXAPI TixHandlerA(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntimeA *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
);
_TIXENTRY BOOL TIXAPI TixHandlerW(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntimeW *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
);

// Signature of TixHandler() functions

typedef BOOL (TIXAPI *fTixHandlerA)(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntimeA *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
);
typedef BOOL (TIXAPI *fTixHandlerW)(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntimeW *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
);

// Decorated names as emitted by the compiler
#define TIX_ENTRYPOINTA			"_TixHandlerA@12"
#define TIX_ENTRYPOINTW			"_TixHandlerW@12"

#ifdef _UNICODE
#define TixHandler TixHandlerW
#define fTixHandler fTixHandlerW
#define TIX_ENTRYPOINT TIX_ENTRYPOINTW
#else
#define TixHandler TixHandlerA
#define fTixHandler fTixHandlerA
#define TIX_ENTRYPOINT TIX_ENTRYPOINTA
#endif

//===========================================================================
// Custom entry point declarations
//
// Custom entry points may be provided by clients in their extension DLL.
// Names are up to the client, but they must use the prototype shown below.
//
// If the custom entry point is called CustomName in the client's source
// code and the Run DLL action in the project, then the installer will try
// to find the following names in the DLL:
//
//	_CustomNameA@12				(ANSI installer)
//	_CustomNameW@12				(Unicode installer)
//===========================================================================

// Prototypes of custom entry points; clients must use their own names
// instead and if possible provide both ANSI (A) and Unicode (W) versions.
//
// The entry point must return ERROR_SUCCESS (0) on success, or a nonzero
// error code on failure. These return values are used to set the
// LastExitCode variable in the installer.

typedef DWORD (TIXAPI *fTixCustomFuncA)(
	iTixRuntimeA *pTixRuntime,	// [In] Runtime engine interface
	int nArgCount,				// [In] Number of arguments
	const ACHAR **ppszArguments	// [In] Actual arguments
);
typedef DWORD (TIXAPI *fTixCustomFuncW)(
	iTixRuntimeW *pTixRuntime,	// [In] Runtime engine interface
	int nArgCount,				// [In] Number of arguments
	const WCHAR **ppszArguments	// [In] Actual arguments
);

// Decorated name patterns as emitted by the compiler
#define TIX_CUSTOMFUNCA			"_%sA@12"
#define TIX_CUSTOMFUNCW			"_%sW@12"

#ifdef _UNICODE
#define fTixCustomFunc fTixCustomFuncW
#define TIX_CUSTOMFUNC TIX_CUSTOMFUNCW
#else
#define fTixCustomFunc fTixCustomFuncA
#define TIX_CUSTOMFUNC TIX_CUSTOMFUNCA
#endif

//===========================================================================
// Message-related definitions
//===========================================================================

//-----	Message parameter block
//
// TIN2 note: pTixRuntime has become a TixHandler() parameter.

struct cTixMessage
{
	DWORD dwMessageID;			// [In] Message identifier; see TIXMSG_xxx
	DWORD dwMessageFlags;		// [In] Message flags; see TIXFLG_xxx
	DWORD dwSessionType;		// [In] Session type; see TIXSES_xxx
	LONG lResult;				// [Out] Result code; see TIXRET_xxx
	LPARAM lCookie;				// [In/out] Client cookie
};

//-----	cTixRegistrationInfo
//
// Represents customer registration information and is passed in through
// the pExtra parameter of TixHandler() for some messages.

#define TIX_REGINFO_MAXSIZE		64

typedef struct cTixRegistrationInfoA
{
	ACHAR szField1[TIX_REGINFO_MAXSIZE];	// (RegisteredOwner)
	ACHAR szField2[TIX_REGINFO_MAXSIZE];	// (RegisteredOrganization)
	ACHAR szField3[TIX_REGINFO_MAXSIZE];
	LPARAM lCookie;							// Registration-specific cookie
} cTixRegistrationInfoA;

typedef struct cTixRegistrationInfoW
{
	WCHAR szField1[TIX_REGINFO_MAXSIZE];	// (RegisteredOwner)
	WCHAR szField2[TIX_REGINFO_MAXSIZE];	// (RegisteredOrganization)
	WCHAR szField3[TIX_REGINFO_MAXSIZE];
	LPARAM lCookie;							// Registration-specific cookie
} cTixRegistrationInfoW;

#ifdef _UNICODE
#define cTixRegistrationInfo cTixRegistrationInfoW
#else
#define cTixRegistrationInfo cTixRegistrationInfoA
#endif

//----- Message codes (cTixMessage::dwMessageID)

#define TIXMSG_GETVERSION					0
#define TIXMSG_INITIALIZE					1
#define TIXMSG_TERMINATE					2
#define TIXMSG_CHECK_REQUIREMENTS			3
#define TIXMSG_INIT_REGISTRATION			4	// pExtra = cTixRegistrationInfo*
#define TIXMSG_CHECK_REGISTRATION			5	// pExtra = cTixRegistrationInfo*
#define TIXMSG_APPLY_REGISTRATION			6	// pExtra = cTixRegistrationInfo*
#define TIXMSG_START_PROCESSING				7
#define TIXMSG_END_PROCESSING				8
#define TIXMSG_ABORT_PROCESSING				9
#define TIXMSG_INIT_OPTIONS					10
#define TIXMSG_CHECK_OPTIONS				11
#define TIXMSG_START_FILE_INSTALL			12
#define TIXMSG_END_FILE_INSTALL				13
#define TIXMSG_START_FILE_REGISTRATION		14
#define TIXMSG_END_FILE_REGISTRATION		15
#define TIXMSG_START_SHORTCUT_INSTALL		16
#define TIXMSG_END_SHORTCUT_INSTALL			17
#define TIXMSG_START_SYSTEM_INSTALL			18
#define TIXMSG_END_SYSTEM_INSTALL			19
#define TIXMSG_START_FILE_REMOVAL			20
#define TIXMSG_END_FILE_REMOVAL				21
#define TIXMSG_START_FILE_UNREGISTRATION	22
#define TIXMSG_END_FILE_UNREGISTRATION		23
#define TIXMSG_START_SHORTCUT_REMOVAL		24
#define TIXMSG_END_SHORTCUT_REMOVAL			25
#define TIXMSG_START_SYSTEM_REMOVAL			26
#define TIXMSG_END_SYSTEM_REMOVAL			27
#define TIXMSG_CHECK_UNINSTALL				28

//-----	Message flags (cTixMessage::dwMessageFlags)
//
// Message flags may be OR-ed together and are subdivided into generic and
// message-specific flags. When testing if a given flag is set, you
// should use bitwise AND (&) and *not* a straight == comparison.

#define TIXFLG_MASK_GENERIC		0x0000FFFF	// Generic message flags
#define TIXFLG_MASK_SPECIFIC	0xFFFF0000	// Message-specific flags

//-----	Session type identifiers (cTixMessage::dwSessionType)
//
// Tarma Setup can run in various guises; the session type indicates in which
// capacity the current session takes place.

#define TIXSES_PREFLIGHT		0		// Preflight test (Tarma Workshop)
#define TIXSES_INSTALL			1		// Regular installation run
#define TIXSES_UNINSTALL		2		// Regular uninstallation run
#define TIXSES_REGISTER			3		// Post-boot registration run
#define TIXSES_REPAIR			4		// Installation repair run [TIN3]

//-----	Result codes (cTixMessage::lResult)
//
// Applicable result codes depend on the message type. In general, positive
// values denote "success" in some form, whereas negative values indicate
// an error while handling the message.

#define TIXRET_NOTHANDLED		0		// Message was not handled
#define TIXRET_SUCCESS			1		// OK; continue
#define TIXRET_REJECT			2		// OK, but do not continue
#define TIXRET_NOREGISTRATION	3		// OK, but no registration page
#define TIXRET_ERROR			(-1)	// Extension DLL detected an error
#define TIXRET_INTERNALERROR	(-2)	// TIX detected an error
#define TIXRET_ABORT			(-3)	// Extension DLL experienced fatal error
#define TIXRET_EXCEPTION		(-4)	// TIX caught an exception from the DLL


#ifdef __cplusplus
}		// extern "C"
#endif

#pragma pack(pop)

#endif	// __TIX_H
