/****************************************************************************
	$Header: /Dev/Tin3/InstallDir/Tix/TixDll.c 19    30/08/06 15:20 Dave $

	Copyright (c) 1990-2005 Tarma Software Research. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	2002-10-24 10:13

	Implementation of a sample/test extension DLL for TIN Setup.

	NOTE: For best results, view with tab stops set at every 4 positions.

	THE CONTENTS OF THIS DOCUMENT ARE CONFIDENTIAL AND PROPRIETARY TO
	TARMA SOFTWARE RESEARCH PTY LTD, AND MAY NOT BE REPRODUCED, PUBLISHED,
	OR DISCLOSED TO OTHERS WITHOUT COMPANY AUTHORISATION.

	Use of this document is allowed to registered users of Tarma Installer
	and solely for the purpose of creating Tarma Installer Extension DLLs.
****************************************************************************/

// Precompiled headers
#include "TixPrecomp.h"
#pragma hdrstop					// End of precompiled headers

// Project headers
#define _TIX_BUILD				// We're implementing the extension DLL
#include "Tix.h"
#include "TixHandlers.h"
#include "TixRuntime.h"
#include "TixUtils.h"

/*===========================================================================
	Configuration options

	Define or undefine the following macros as desired.

	TIX_NOCRT
		Define to build a DLL that does not depend on the MSVC CRT. This
		makes a much smaller DLL, but you can use only Win32 API functions
		and NOT any C/C++ library functions.

	Furthermore, check the file for "TODO" comments that indicate where you
	should add your own code.

	Recommended compiler options (MSVC):
		/W4		- Maximum warnings
		/O1		- Minimize size
		Link with single-threaded static runtime library
===========================================================================*/

#define TIX_NOCRT	// #undef to link with MSVC CRT

//===========================================================================
// Declarations and definitions
//===========================================================================

HINSTANCE g_hTixDll;

// Table of handler functions
static _fTixOnHandle g_pfHandlers[] =
{
	NULL,							// TIXMSG_GETVERSION
	_TixOnInitialize,				// TIXMSG_INITIALIZE
	_TixOnTerminate,				// TIXMSG_TERMINATE
	_TixOnCheckRequirements,		// TIXMSG_CHECK_REQUIREMENTS
	_TixOnInitRegistration,			// TIXMSG_INIT_REGISTRATION
	_TixOnCheckRegistration,		// TIXMSG_CHECK_REGISTRATION
	_TixOnApplyRegistration,		// TIXMSG_APPLY_REGISTRATION
	_TixOnStartProcessing,			// TIXMSG_START_PROCESSING
	_TixOnEndProcessing,			// TIXMSG_END_PROCESSING
	_TixOnAbortProcessing,			// TIXMSG_ABORT_PROCESSING
	_TixOnInitOptions,				// TIXMSG_INIT_OPTIONS
	_TixOnCheckOptions,				// TIXMSG_CHECK_OPTIONS
	_TixOnStartFileInstall,			// TIXMSG_START_FILE_INSTALL
	_TixOnEndFileInstall,			// TIXMSG_END_FILE_INSTALL
	_TixOnStartFileRegistration,	// TIXMSG_START_FILE_REGISTRATION
	_TixOnEndFileRegistration,		// TIXMSG_END_FILE_REGISTRATION
	_TixOnStartShortcutInstall,		// TIXMSG_START_SHORTCUT_INSTALL
	_TixOnEndShortcutInstall,		// TIXMSG_END_SHORTCUT_INSTALL
	_TixOnStartSystemInstall,		// TIXMSG_START_SYSTEM_INSTALL
	_TixOnEndSystemInstall,			// TIXMSG_END_SYSTEM_INSTALL
	_TixOnStartFileRemoval,			// TIXMSG_START_FILE_REMOVAL
	_TixOnEndFileRemoval,			// TIXMSG_END_FILE_REMOVAL
	_TixOnStartFileUnregistration,	// TIXMSG_START_FILE_UNREGISTRATION
	_TixOnEndFileUnregistration,	// TIXMSG_END_FILE_UNREGISTRATION
	_TixOnStartShortcutRemoval,		// TIXMSG_START_SHORTCUT_REMOVAL
	_TixOnEndShortcutRemoval,		// TIXMSG_END_SHORTCUT_REMOVAL
	_TixOnStartSystemRemoval,		// TIXMSG_START_SYSTEM_REMOVAL
	_TixOnEndSystemRemoval,			// TIXMSG_END_SYSTEM_REMOVAL
	_TixOnCheckUninstall,			// TIXMSG_CHECK_UNINSTALL
};

//===========================================================================
// TIX handler entry point
//===========================================================================

_TIXENTRY BOOL TIXAPI TixHandler(cTixMessage *pMessage,
	iTixRuntime *pTixRuntime, void *pExtra)
{
	BOOL bHandled = FALSE;

	// The following should never happen, but better safe than sorry...
	if (pMessage == NULL || pTixRuntime == NULL)
	{
		TIX_LOG((_T("TixHandler: NULL message pointers\r\n")));
		return FALSE;
	}

	TIX_LOG((_T("==> TixHandler: Msg=%u (%s), Ses=%u, Flags=%08X, Cookie=%08lX, Extra=%08lX\r\n"),
		pMessage->dwMessageID, TIX_MSGNAME(pMessage->dwMessageID),
		pMessage->dwSessionType, pMessage->dwMessageFlags,
		pMessage->lCookie, (LPARAM)pExtra));

	// Use a lookup table for the message handlers
	if (pMessage->dwMessageID == TIXMSG_GETVERSION)
	{
		// Must be handled, but always in the same manner.
		pMessage->lResult = __TIXVER__;
		bHandled = TRUE;
	}
	else if (pMessage->dwMessageID < __countof(g_pfHandlers))
	{
		if (g_pfHandlers[pMessage->dwMessageID] != NULL)
		{
			bHandled = (*g_pfHandlers[pMessage->dwMessageID])(pMessage,
				pTixRuntime, pExtra);
			if (pMessage->lResult == TIXRET_NOTHANDLED)
				bHandled = FALSE;
		}
		else
			TIX_LOG((_T("TixHandler: NULL handler for message %u\r\n"),
				pMessage->dwMessageID));
	}
	else
	{
		// Unknown (new?) messages
		TIX_LOG((_T("TixHandler: WARNING - Ignoring unknown message %u\r\n"),
			pMessage->dwMessageID));
	}

	TIX_LOG((_T("<== TixHandler: Msg=%u (%s) result=%ld\r\n"),
		pMessage->dwMessageID, TIX_MSGNAME(pMessage->dwMessageID),
		pMessage->lResult));
	return bHandled;
}

//===========================================================================
// DLL main entry point
//===========================================================================

BOOL __stdcall DllMain(HINSTANCE hModule, DWORD dwReason, LPVOID pReserved)
{
	UNUSED(pReserved);

	// TODO: Add your own initialization as needed
	switch (dwReason)
	{
	case DLL_PROCESS_ATTACH:
		g_hTixDll = hModule;	// Remember our module handle
		TIX_OPENLOG();
		TIX_LOG((_T("DLL_PROCESS_ATTACH seen\r\n")));
		break;

	case DLL_PROCESS_DETACH:
		TIX_LOG((_T("DLL_PROCESS_DETACH seen\r\n")));
		TIX_CLOSELOG();
		break;

	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	default:
		break;
	}
	return TRUE;
}

//===========================================================================
// Implementation helpers
//===========================================================================

// Avoid silly large alignment sizes
#if _MSC_VER == 1200
#pragma comment(linker, "/opt:nowin98")
#endif

//---------------------------------------------------------------------------
// Predefined entry point used by Microsoft LINK

#ifdef TIX_NOCRT

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hModule, DWORD dwReason,
	LPVOID pReserved)
{
	// Run the user's entry point
	return DllMain(hModule, dwReason, pReserved);
}

#endif	// TIX_NOCRT
