/****************************************************************************
	$Header: /Dev/Tin3/InstallDir/Tix/TixDll_Switch.c 2     16-02-06 18:21 Ron $

	Copyright (c) 1990-2005 Tarma Software Research. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	2002-10-24 10:13

	Implementation of a sample/test extension DLL for TIN Setup.

	NOTE: For best results, view with tab stops set at every 4 positions.

	THE CONTENTS OF THIS DOCUMENT ARE CONFIDENTIAL AND PROPRIETARY TO
	TARMA SOFTWARE RESEARCH PTY LTD, AND MAY NOT BE REPRODUCED, PUBLISHED,
	OR DISCLOSED TO OTHERS WITHOUT COMPANY AUTHORISATION.

	Use of this document is allowed to registered users of Tarma Installer
	and solely for the purpose of creating Tarma Installer Extension DLLs.
****************************************************************************/

//-----	Precompiled headers

#if _MSC_VER >= 1200
#pragma warning(push)
#endif

// Pragmas to avoid complaints on the Windows header files with /W4
#pragma warning(disable:4115)	// Named type definition in parentheses
#pragma warning(disable:4201)	// Non-standard: nameless struct/union
#pragma warning(disable:4214)	// Non-standard: bit field other than int
#pragma warning(disable:4255)	// No function prototype: converting () to (void)
#pragma warning(disable:4514)	// Unreferenced inline function removed
#pragma warning(disable:4668)	// Name not defined as preprocessor name
#pragma warning(disable:4711)	// Function selected for automatic inline
#pragma warning(disable:4820)	// n bytes padding added after struct member
#pragma warning(disable:4917)	// GUID must be associated with class, interface, namespace

// Windows headers
#define STRICT
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#if _MSC_VER >= 1200
#pragma warning(pop)
#endif

#pragma warning(disable:4514)	// Unreferenced inline function removed

#pragma hdrstop					// End of precompiled headers

//-----	Project headers

#define _TIX_BUILD				// We're implementing the extension DLL
#include "Tix.h"
#include "TixRuntime.h"
#include "TixUtils.h"

/*===========================================================================
	Configuration options

	Define or undefine the following macros as desired.

	TIX_NOCRT
		Define to build a DLL that does not depend on the MSVC CRT. This
		makes a much smaller DLL, but you can use only Win32 API functions
		and NOT any C/C++ library functions.

	Furthermore, check the file for "TODO" comments that indicate where you
	should add your own code.

	Recommended compiler options (MSVC):
		/W4		- Maximum warnings
		/O1		- Minimize size
		Link with single-threaded static runtime library
===========================================================================*/

#define TIX_NOCRT	// #undef to link with MSVC CRT

//===========================================================================
// Declarations and definitions
//===========================================================================

HINSTANCE g_hTixDll;

//===========================================================================
// TIX handler entry point
//===========================================================================

// Implementation & test helpers
#define PREGINFO(ptr)	((cTixRegistrationInfo *)(ptr))

_TIXENTRY BOOL TIXAPI TixHandler(cTixMessage *pMessage,
	iTixRuntime *pTixRuntime, void *pExtra)
{
	// The following should never happen, but better safe than sorry...
	if (pMessage == NULL || pTixRuntime == NULL)
	{
		TIX_LOG((_T("TixHandler: NULL message pointers\r\n")));
		return FALSE;
	}

	TIX_LOG((_T("TixHandler: Msg=%u (%s), Ses=%u, Flags=%08X, Cookie=%08X, Extra=%08X\r\n"),
		pMessage->dwMessageID, TIX_MSGNAME(pMessage->dwMessageID),
		pMessage->dwSessionType, pMessage->dwMessageFlags,
		pMessage->lCookie, (LPARAM)pExtra));

	// TODO: Add message handling for messages that you're interested in.
	// Return TRUE from the switch case branch to indicate that the message
	// has been handled. If the handler falls through, it will return FALSE.

	switch (pMessage->dwMessageID)
	{
	case TIXMSG_GETVERSION:
		// Must be handled, but always in the same manner.
		pMessage->lResult = __TIXVER__;
		TIX_LOG((_T("TixHandler: TIXMSG_GETVERSION returns version %ld\r\n"),
			pMessage->lResult));
		return TRUE;

	case TIXMSG_INITIALIZE:
		// Pretend success
		pMessage->lResult = TIXRET_SUCCESS;
		return TRUE;

	case TIXMSG_TERMINATE:
		// Pretend success
		pMessage->lCookie = 0;
		pMessage->lResult = TIXRET_SUCCESS;
		return TRUE;

	case TIXMSG_INIT_REGISTRATION:
		if (pExtra)
		{
			TIX_LOG((_T("TixHandler: TIXMSG_INIT_REGISTRATION =>\r\n")
				_T("- Field 1=%s\r\n")
				_T("- Field 2=%s\r\n")
				_T("- Field 3=%s\r\n"),
				PREGINFO(pExtra)->szField1,
				PREGINFO(pExtra)->szField2,
				PREGINFO(pExtra)->szField3));

			// TODO: Add your own initialization, then return TRUE
			pMessage->lResult = TIXRET_SUCCESS;
//?			pMessage->lResult = TIXRET_NOREGISTRATION;	// No registration after all
			return TRUE;
		}
		break;

	case TIXMSG_CHECK_REGISTRATION:
		if (pExtra)
		{
			TIX_LOG((_T("TixHandler: TIXMSG_CHECK_REGISTRATION =>\r\n")
				_T("- Field 1=%s\r\n")
				_T("- Field 2=%s\r\n")
				_T("- Field 3=%s\r\n"),
				PREGINFO(pExtra)->szField1,
				PREGINFO(pExtra)->szField2,
				PREGINFO(pExtra)->szField3));

			// TODO: Add your own checks, then return TRUE
			pMessage->lResult = TIXRET_SUCCESS;	// Registration info OK
//?			pMessage->lResult = TIXRET_REJECT;	// Registration info invalid
			return TRUE;
		}
		break;

	case TIXMSG_APPLY_REGISTRATION:
		if (pExtra)
		{
			TIX_LOG((_T("TixHandler: TIXMSG_APPLY_REGISTRATION =>\r\n")
				_T("- Field 1=%s\r\n")
				_T("- Field 2=%s\r\n")
				_T("- Field 3=%s\r\n"),
				PREGINFO(pExtra)->szField1,
				PREGINFO(pExtra)->szField2,
				PREGINFO(pExtra)->szField3));

			// TODO: Perform registration, then return TRUE
			pMessage->lResult = TIXRET_SUCCESS;
			return TRUE;
		}
		break;

	case TIXMSG_CHECK_REQUIREMENTS:
		// TODO: Check your target system requirements
		pMessage->lResult = TIXRET_SUCCESS;	// Target system OK
//?		pMessage->lResult = TIXRET_REJECT;	// Target system doesn't pass
		return TRUE;

	//-----	Overall processing boundaries
	case TIXMSG_START_PROCESSING:
	case TIXMSG_END_PROCESSING:
	case TIXMSG_ABORT_PROCESSING:
		// TODO: These messages wrap the actual processing. You may only
		// modify the target system between TIXMSG_START_PROCESSING and
		// either TIXMSG_END_PROCESSING (regular end) or
		// TIXMSG_ABORT_PROCESSING (failure or user cancellation).
		pMessage->lResult = TIXRET_SUCCESS;
		return TRUE;

	//-----	Installation messages
	case TIXMSG_INIT_OPTIONS:
	case TIXMSG_CHECK_OPTIONS:
	case TIXMSG_START_FILE_INSTALL:
	case TIXMSG_END_FILE_INSTALL:
	case TIXMSG_START_FILE_REGISTRATION:
	case TIXMSG_END_FILE_REGISTRATION:
	case TIXMSG_START_SHORTCUT_INSTALL:
	case TIXMSG_END_SHORTCUT_INSTALL:
	case TIXMSG_START_SYSTEM_INSTALL:
	case TIXMSG_END_SYSTEM_INSTALL:
		// TODO: Add handlers for whatever you're interested in
		break;

	//-----	Uninstallation messages
	case TIXMSG_START_FILE_REMOVAL:
	case TIXMSG_END_FILE_REMOVAL:
	case TIXMSG_START_FILE_UNREGISTRATION:
	case TIXMSG_END_FILE_UNREGISTRATION:
	case TIXMSG_START_SHORTCUT_REMOVAL:
	case TIXMSG_END_SHORTCUT_REMOVAL:
	case TIXMSG_START_SYSTEM_REMOVAL:
	case TIXMSG_END_SYSTEM_REMOVAL:
		// TODO: Add handlers for whatever you're interested in
		break;

	default:
		// Unknown (new?) messages
		TIX_LOG((_T("TixHandler: WARNING - Ignoring unknown message\r\n")));
		break;
	}

	// If we get here, the message wasn't handled
	return FALSE;
}


//===========================================================================
// DLL main entry point
//===========================================================================

BOOL __stdcall DllMain(HINSTANCE hModule, DWORD dwReason, LPVOID pReserved)
{
	UNUSED(pReserved);

	// TODO: Add your own initialization as needed
	switch (dwReason)
	{
	case DLL_PROCESS_ATTACH:
		g_hTixDll = hModule;	// Remember our module handle
		TIX_OPENLOG();
		TIX_LOG((_T("DLL_PROCESS_ATTACH seen\r\n")));
		break;

	case DLL_PROCESS_DETACH:
		TIX_LOG((_T("DLL_PROCESS_DETACH seen\r\n")));
		TIX_CLOSELOG();
		break;

	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	default:
		break;
	}
	return TRUE;
}

//===========================================================================
// Implementation helpers
//===========================================================================

// Avoid silly large alignment sizes
#if _MSC_VER == 1200
#pragma comment(linker, "/opt:nowin98")
#endif

//---------------------------------------------------------------------------
// Predefined entry point used by Microsoft LINK

#ifdef TIX_NOCRT

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hModule, DWORD dwReason,
	LPVOID pReserved)
{
	// Run the user's entry point
	return DllMain(hModule, dwReason, pReserved);
}

#endif	// TIX_NOCRT
