/****************************************************************************
	$Header: /Dev/Tin3/InstallDir/Tix/TixHandlers.cpp 5     30/08/06 15:20 Dave $

	Copyright (c) 1990-2005 Tarma Software Research. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	01-12-2005 10:42

	Implementation of TIX message handlers.

	NOTE: For best results, view with tab stops set at every 4 positions.

	THE CONTENTS OF THIS DOCUMENT ARE CONFIDENTIAL AND PROPRIETARY TO
	TARMA SOFTWARE RESEARCH PTY LTD, AND MAY NOT BE REPRODUCED, PUBLISHED,
	OR DISCLOSED TO OTHERS WITHOUT COMPANY AUTHORISATION.

	Use of this document is allowed to registered users of Tarma Installer
	and solely for the purpose of creating Tarma Installer Extension DLLs.
****************************************************************************/

// Precompiled headers
#include "TixPrecomp.h"
#pragma hdrstop					// End of precompiled headers

// Project headers
#define _TIX_BUILD				// We're implementing the extension DLL
#include "Tix.h"
#include "TixHandlers.h"
#include "TixRuntime.h"
#include "TixUtils.h"

//---------------------------------------------------------------------------
// TIXMSG_INITIALIZE

BOOL TIXAPI _TixOnInitialize(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform your own initialization. You may store a session cookie
	// in pMessage->lCookie if you need to maintain state information
	// between messages.

	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_TERMINATE

BOOL TIXAPI _TixOnTerminate(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform your own cleanup. If you stored any session state via
	// pMessage->lCookie, this is the time to release it.

	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_CHECK_REQUIREMENTS

BOOL TIXAPI _TixOnCheckRequirements(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Check additional target system requirements, beyond those
	// performed by Setup.exe itself.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
//?	pMessage->lResult = TIXRET_REJECT;	// Target system doesn't pass
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_INIT_REGISTRATION

BOOL TIXAPI _TixOnInitRegistration(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] cTixRegistrationInfo*
)
{
	cTixRegistrationInfo *pInfo = (cTixRegistrationInfo *)pExtra;

	UNUSED(pTixRuntime);

	TIX_LOG((_T("OnInitRegistration:\r\n")
		_T("- Field 1=%s\r\n")
		_T("- Field 2=%s\r\n")
		_T("- Field 3=%s\r\n"),
		pInfo->szField1, pInfo->szField2, pInfo->szField3));

	// TODO: Initialize the user registration info. It is pre-initialized
	// by the installer based on the values of the RegField1-3 variables.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
//?	pMessage->lResult = TIXRET_NOREGISTRATION;	// To prevent registration page
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_CHECK_REGISTRATION

BOOL TIXAPI _TixOnCheckRegistration(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] cTixRegistrationInfo*
)
{
	cTixRegistrationInfo *pInfo = (cTixRegistrationInfo *)pExtra;

	UNUSED(pTixRuntime);

	TIX_LOG((_T("OnCheckRegistration:\r\n")
		_T("- Field 1=%s\r\n")
		_T("- Field 2=%s\r\n")
		_T("- Field 3=%s\r\n"),
		pInfo->szField1, pInfo->szField2, pInfo->szField3));

	// TODO: Check the user registration info and determine if the
	// installation may proceed. If not, you should tell the user with
	// TixMessageBox(). This message may appear repeatedly, until the
	// registration info is validated.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
//?	pMessage->lResult = TIXRET_REJECT;	// Registration info invalid
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_APPLY_REGISTRATION

BOOL TIXAPI _TixOnApplyRegistration(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] cTixRegistrationInfo*
)
{
	cTixRegistrationInfo *pInfo = (cTixRegistrationInfo *)pExtra;

	UNUSED(pTixRuntime);

	TIX_LOG((_T("OnApplyRegistration:\r\n")
		_T("- Field 1=%s\r\n")
		_T("- Field 2=%s\r\n")
		_T("- Field 3=%s\r\n"),
		pInfo->szField1, pInfo->szField2, pInfo->szField3));

	// TODO: Store the registration info if you don't want to use the
	// default Setup.exe action of storing it in the registry.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_PROCESSING

BOOL TIXAPI _TixOnStartProcessing(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Any actions that you want to perform as soon as the
	// installation or removal gets underway.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_PROCESSING

BOOL TIXAPI _TixOnEndProcessing(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Any actions that you want to perform at the very end of the
	// installation or removal.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_ABORT_PROCESSING

BOOL TIXAPI _TixOnAbortProcessing(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform emergency cleanup if the session is aborted for some
	// reason. In that case, _TixOnEndProcessing() is not called.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_INIT_OPTIONS

BOOL TIXAPI _TixOnInitOptions(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Initialize any installation options, for example the default
	// installation folder paths or feature states.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_CHECK_OPTIONS

BOOL TIXAPI _TixOnCheckOptions(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Check the installation options, beyond what Setup.exe already
	// checks (write access, disk space). If you prevent installation,
	// you should use TixMessageBox() to tell the user why.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
//?	pMessage->lResult = TIXRET_REJECT;	// Invalid options
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_FILE_INSTALL

BOOL TIXAPI _TixOnStartFileInstall(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// file installation.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_FILE_INSTALL

BOOL TIXAPI _TixOnEndFileInstall(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// file installation.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_FILE_REGISTRATION

BOOL TIXAPI _TixOnStartFileRegistration(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// file registration.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_FILE_REGISTRATION

BOOL TIXAPI _TixOnEndFileRegistration(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// file registration.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_SHORTCUT_INSTALL

BOOL TIXAPI _TixOnStartShortcutInstall(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// shortcut installation.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_SHORTCUT_INSTALL

BOOL TIXAPI _TixOnEndShortcutInstall(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// shortcut installation.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_SYSTEM_INSTALL

BOOL TIXAPI _TixOnStartSystemInstall(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// system installation (registry, environment, INI files).

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_SYSTEM_INSTALL

BOOL TIXAPI _TixOnEndSystemInstall(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// system installation (registry, environment, INI files).

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_FILE_REMOVAL

BOOL TIXAPI _TixOnStartFileRemoval(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// file removal.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_FILE_REMOVAL

BOOL TIXAPI _TixOnEndFileRemoval(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// file removal.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_FILE_UNREGISTRATION

BOOL TIXAPI _TixOnStartFileUnregistration(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// file unregistration.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_FILE_UNREGISTRATION

BOOL TIXAPI _TixOnEndFileUnregistration(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// file unregistration.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_SHORTCUT_REMOVAL

BOOL TIXAPI _TixOnStartShortcutRemoval(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// shortcut removal.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_SHORTCUT_REMOVAL

BOOL TIXAPI _TixOnEndShortcutRemoval(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// shortcut removal.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_START_SYSTEM_REMOVAL

BOOL TIXAPI _TixOnStartSystemRemoval(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately before
	// system removal (registry, environment, INI files).

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_END_SYSTEM_REMOVAL

BOOL TIXAPI _TixOnEndSystemRemoval(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Perform any actions that must take place immediately after
	// system removal (registry, environment, INI files).

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
	return TRUE;
}

//---------------------------------------------------------------------------
// TIXMSG_CHECK_UNINSTALL

BOOL TIXAPI _TixOnCheckUninstall(
	cTixMessage *pMessage,		// [In] Message parameters
	iTixRuntime *pTixRuntime,	// [In] Runtime engine interface
	void *pExtra				// [In,opt] Additional data
)
{
	UNUSED(pTixRuntime);
	UNUSED(pExtra);

	// TODO: Check if the uninstallation may start. If you prevent
	// uninstallation, you should use TixMessageBox() to tell the user why.

	pMessage->lResult = TIXRET_NOTHANDLED;
//?	pMessage->lResult = TIXRET_SUCCESS;	// Message was handled successfully
//?	pMessage->lResult = TIXRET_REJECT;	// Don't uninstall
	return TRUE;
}

