/****************************************************************************
	$Header: /Dev/Tin3/InstallDir/Tix/TixRuntime.h 18    30/08/06 15:20 Dave $

	Copyright (c) 1990-2005 Tarma Software Research. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	2002-10-23 11:30

	Declaration of the Tarma Setup runtime engine interface.

	NOTE: For best results, view with tab stops set at every 4 positions.

	THE CONTENTS OF THIS DOCUMENT ARE CONFIDENTIAL AND PROPRIETARY TO
	TARMA SOFTWARE RESEARCH PTY LTD, AND MAY NOT BE REPRODUCED, PUBLISHED,
	OR DISCLOSED TO OTHERS WITHOUT COMPANY AUTHORISATION.

	Use of this document is allowed to registered users of Tarma Installer
	and solely for the purpose of creating Tarma Installer Extension DLLs.
****************************************************************************/

#ifndef __TIXRUNTIME_H
#define __TIXRUNTIME_H

#pragma once

#ifndef __TIXDEFS_H
#include "TixDefs.h"	// Base declarations
#endif

#pragma pack(push,4)	// Use 4-byte alignment throughout

#ifdef __cplusplus
extern "C" {
#endif

//===========================================================================
// Preliminary definitions
//===========================================================================

//-----	Forward declarations

typedef struct iTixRuntimeA iTixRuntimeA;
typedef struct iTixRuntimeW iTixRuntimeW;

#ifdef _UNICODE
#define iTixRuntime iTixRuntimeW
#else
#define iTixRuntime iTixRuntimeA
#endif

//===========================================================================
// Interface to the Tarma Setup engine
//
// This interface is implemented by Tarma Setup. Its purpose is to allow
// extension DLLs access to Tarma Setup's runtime engine.
//
// The interface mechanism is similar to a COM interface, but it does not
// use COM. It is entirely implemented within Tarma Setup and does *not*
// require any COM initialization. However, its terminology is deliberately
// chosen to mirror COM programming practices in order to flatten the
// learning curve for developers.
//
// A pointer to the object implementing this interface is passed to the
// extension DLL TixHandler() entry point as pMessage->pTixRuntime.
//===========================================================================

//---------------------------------------------------------------------------
// TIN3 NOTE: the following have changed since Tin2; failure/cancellation
// status codes are now negative.

#define TIXSTAT_CANCELLED			(-2)	// Session is cancelled by the user
#define TIXSTAT_FAILED				(-1)	// Session failed due to an error
#define TIXSTAT_NONE				0		// Session not started yet
#define TIXSTAT_INPROGRESS			1		// Session is in progress
#define TIXSTAT_COMPLETE			2		// Session is complete (with success)

//---------------------------------------------------------------------------
// GetFileInstallStatus() result codes

#define TIXFILE_NOTINSTALLED		0		// Not installed or not found
	// Success codes are > 0
#define TIXFILE_SUCCESSNEW			1		// Succeeded: new file installed
#define TIXFILE_SUCCESSUPDATE		2		// Succeeded: current file updated
#define TIXFILE_SUCCESSEXISTING		3		// Succeeded: existing target was up to date
	// Failure codes are < 0
#define TIXFILE_ERROR				(-1)	// Failed: error while installing file

//---------------------------------------------------------------------------
// GetInternalFlags() return flags

#define TIXFLAGS_REBOOT			0x00000001	// Reboot required
#define TIXFLAGS_POSTBOOTREG	0x00000002	// Post-boot file reg required

#ifdef __cplusplus

//---------------------------------------------------------------------------
// C++ version of the interface

struct iTixRuntimeA
{
	virtual DWORD TIXCALLAPI GetVersion() = 0;
	virtual HWND TIXCALLAPI GetWindow() = 0;
	virtual BOOL TIXCALLAPI IsNT() = 0;
	virtual BOOL TIXCALLAPI IsAdministrator() = 0;
	virtual BOOL TIXCALLAPI IsSilentInstall() = 0;
	virtual BOOL TIXCALLAPI ResolveSymbols(const ACHAR *pszString,
		ACHAR *pszResult, UINT uResultCount) = 0;
	virtual BOOL TIXCALLAPI GetVersionString(const ACHAR *pszName,
		ACHAR *pszResult, UINT uResultCount) = 0;
	virtual BOOL TIXCALLAPI SetSymbol(const ACHAR *pszSymbolName,
		const ACHAR *pszNewValue) = 0;
	virtual WORD TIXCALLAPI GetUILangID() = 0;
	virtual WORD TIXCALLAPI GetUserLangID() = 0;
	virtual int TIXCALLAPI GetSessionStatus() = 0;
	virtual void TIXCALLAPI FailInstall() = 0;
	virtual void TIXCALLAPI LogMessage(const ACHAR *pszFormat, ...) = 0;
	virtual void TIXCALLAPI SetReboot(BOOL bFlag) = 0;
	virtual void TIXCALLAPI SetPostBootRegistration(BOOL bFlag) = 0;
	virtual BOOL TIXCALLAPI CheckFileDelete(const ACHAR *pszFilePath) = 0;
	virtual int TIXCALLAPI GetFileInstallStatus(const ACHAR *pszFilePath) = 0;
	virtual void TIXCALLAPI FailInstallEx(BOOL bCleanup) = 0;
	virtual void TIXCALLAPI UpdateFeatures() = 0;
	virtual DWORD TIXCALLAPI GetInternalFlags() = 0;
};

struct iTixRuntimeW
{
	virtual DWORD TIXCALLAPI GetVersion() = 0;
	virtual HWND TIXCALLAPI GetWindow() = 0;
	virtual BOOL TIXCALLAPI IsNT() = 0;
	virtual BOOL TIXCALLAPI IsAdministrator() = 0;
	virtual BOOL TIXCALLAPI IsSilentInstall() = 0;
	virtual BOOL TIXCALLAPI ResolveSymbols(const WCHAR *pszString,
		WCHAR *pszResult, UINT uResultCount) = 0;
	virtual BOOL TIXCALLAPI GetVersionString(const WCHAR *pszName,
		WCHAR *pszResult, UINT uResultCount) = 0;
	virtual BOOL TIXCALLAPI SetSymbol(const WCHAR *pszSymbolName,
		const WCHAR *pszNewValue) = 0;
	virtual WORD TIXCALLAPI GetUILangID() = 0;
	virtual WORD TIXCALLAPI GetUserLangID() = 0;
	virtual int TIXCALLAPI GetSessionStatus() = 0;
	virtual void TIXCALLAPI FailInstall() = 0;
	virtual void TIXCALLAPI LogMessage(const WCHAR *pszFormat, ...) = 0;
	virtual void TIXCALLAPI SetReboot(BOOL bFlag) = 0;
	virtual void TIXCALLAPI SetPostBootRegistration(BOOL bFlag) = 0;
	virtual BOOL TIXCALLAPI CheckFileDelete(const WCHAR *pszFilePath) = 0;
	virtual int TIXCALLAPI GetFileInstallStatus(const WCHAR *pszFilePath) = 0;
	virtual void TIXCALLAPI FailInstallEx(BOOL bCleanup) = 0;
	virtual void TIXCALLAPI UpdateFeatures() = 0;
	virtual DWORD TIXCALLAPI GetInternalFlags() = 0;
};

#else	// Not __cplusplus

//---------------------------------------------------------------------------
// Plain C version of the interface
//
// The syntax to call interface methods is as follows:
//
//	pTixRuntime->lpVtbl->method_name(pTixRuntime, other_params);

typedef struct __iTixRuntimeVtblA
{
	// Function pointers to match C++ vtable layout
	DWORD (TIXCALLAPI *GetVersion)(iTixRuntimeA *pThis);
	HWND (TIXCALLAPI *GetWindow)(iTixRuntimeA *pThis);
	BOOL (TIXCALLAPI *IsNT)(iTixRuntimeA *pThis);
	BOOL (TIXCALLAPI *IsAdministrator)(iTixRuntimeA *pThis);
	BOOL (TIXCALLAPI *IsSilentInstall)(iTixRuntimeA *pThis);
	BOOL (TIXCALLAPI *ResolveSymbols)(iTixRuntimeA *pThis,
		const ACHAR *pszString, ACHAR *pszResult, UINT uResultCount);
	BOOL (TIXCALLAPI *GetVersionString)(iTixRuntimeA *pThis,
		const ACHAR *pszName, ACHAR *pszResult, UINT uResultCount);
	BOOL (TIXCALLAPI *SetSymbol)(iTixRuntimeA *pThis,
		const ACHAR *pszSymbolName, const ACHAR *pszNewValue);
	WORD (TIXCALLAPI *GetUILangID)(iTixRuntimeA *pThis);
	WORD (TIXCALLAPI *GetUserLangID)(iTixRuntimeA *pThis);
	int (TIXCALLAPI *GetSessionStatus)(iTixRuntimeA *pThis);
	void (TIXCALLAPI *FailInstall)(iTixRuntimeA *pThis);
	void (TIXCALLAPI *LogMessage)(iTixRuntimeA *pThis,
		const ACHAR *pszFormat, ...);
	void (TIXCALLAPI *SetReboot)(iTixRuntimeA *pThis, BOOL bFlag);
	void (TIXCALLAPI *SetPostBootRegistration)(iTixRuntimeA *pThis, BOOL bFlag);
	BOOL (TIXCALLAPI *CheckFileDelete)(iTixRuntimeA *pThis,
		const ACHAR *pszFilePath);
	int (TIXCALLAPI *GetFileInstallStatus)(iTixRuntimeA *pThis,
		const ACHAR *pszFilePath);
	void (TIXCALLAPI *FailInstallEx)(iTixRuntimeA *pThis, BOOL bCleanup);
	void (TIXCALLAPI *UpdateFeatures)(iTixRuntimeA *pThis);
	DWORD (TIXCALLAPI *GetInternalFlags)(iTixRuntimeA *pThis);
} __iTixRuntimeVtblA;

struct iTixRuntimeA
{
	const struct __iTixRuntimeVtblA * const lpVtbl;
	// MSVC++ compatible layout for vtable pointer location
};

typedef struct __iTixRuntimeVtblW
{
	// Function pointers to match C++ vtable layout
	DWORD (TIXCALLAPI *GetVersion)(iTixRuntimeW *pThis);
	HWND (TIXCALLAPI *GetWindow)(iTixRuntimeW *pThis);
	BOOL (TIXCALLAPI *IsNT)(iTixRuntimeW *pThis);
	BOOL (TIXCALLAPI *IsAdministrator)(iTixRuntimeW *pThis);
	BOOL (TIXCALLAPI *IsSilentInstall)(iTixRuntimeW *pThis);
	BOOL (TIXCALLAPI *ResolveSymbols)(iTixRuntimeW *pThis,
		const WCHAR *pszString, WCHAR *pszResult, UINT uResultCount);
	BOOL (TIXCALLAPI *GetVersionString)(iTixRuntimeW *pThis,
		const WCHAR *pszName, WCHAR *pszResult, UINT uResultCount);
	BOOL (TIXCALLAPI *SetSymbol)(iTixRuntimeW *pThis,
		const WCHAR *pszSymbolName, const WCHAR *pszNewValue);
	WORD (TIXCALLAPI *GetUILangID)(iTixRuntimeW *pThis);
	WORD (TIXCALLAPI *GetUserLangID)(iTixRuntimeW *pThis);
	int (TIXCALLAPI *GetSessionStatus)(iTixRuntimeW *pThis);
	void (TIXCALLAPI *FailInstall)(iTixRuntimeW *pThis);
	void (TIXCALLAPI *LogMessage)(iTixRuntimeW *pThis,
		const WCHAR *pszFormat, ...);
	void (TIXCALLAPI *SetReboot)(iTixRuntimeW *pThis, BOOL bFlag);
	void (TIXCALLAPI *SetPostBootRegistration)(iTixRuntimeW *pThis, BOOL bFlag);
	BOOL (TIXCALLAPI *CheckFileDelete)(iTixRuntimeW *pThis,
		const WCHAR *pszFilePath);
	int (TIXCALLAPI *GetFileInstallStatus)(iTixRuntimeW *pThis,
		const WCHAR *pszFilePath);
	void (TIXCALLAPI *FailInstallEx)(iTixRuntimeW *pThis, BOOL bCleanup);
	void (TIXCALLAPI *UpdateFeatures)(iTixRuntimeW *pThis);
	DWORD (TIXCALLAPI *GetInternalFlags)(iTixRuntimeW *pThis);
} __iTixRuntimeVtblW;

struct iTixRuntimeW
{
	const struct __iTixRuntimeVtblW * const lpVtbl;
	// MSVC++ compatible layout for vtable pointer location
};

#ifdef _UNICODE
#define __iTixRuntimeVtbl __iTixRuntimeVtblW
#else
#define __iTixRuntimeVtbl __iTixRuntimeVtblA
#endif

//---------------------------------------------------------------------------
// Macros to make the C interface somewhat easier to use

#define TIXRT_GetVersion(pThis) \
	(*pThis->lpVtbl->GetVersion)(pThis)

#define TIXRT_GetWindow(pThis) \
	(*pThis->lpVtbl->GetWindow)(pThis)

#define TIXRT_IsNT(pThis) \
	(*pThis->lpVtbl->IsNT)(pThis)

#define TIXRT_IsAdministrator(pThis) \
	(*pThis->lpVtbl->IsAdministrator)(pThis)

#define TIXRT_IsSilentInstall(pThis) \
	(*pThis->lpVtbl->IsSilentInstall)(pThis)

#define TIXRT_ResolveSymbols(pThis, pszString, pszResult, uResultCount) \
	(*pThis->lpVtbl->ResolveSymbols)(pThis, pszString, pszResult, uResultCount)

#define TIXRT_GetVersionString(pThis, pszName, pszResult, uResultCount) \
	(*pThis->lpVtbl->GetVersionString)(pThis, pszResult, uResultCount)

#define TIXRT_SetSymbol(pThis, pszSymbolName, pszNewValue) \
	(*pThis->lpVtbl->SetSymbol)(pThis, pszSymbolName, pszNewValue)

#define TIXRT_GetUILangID(pThis) \
	(*pThis->lpVtbl->GetUILangID)(pThis)

#define TIXRT_GetUserLangID(pThis) \
	(*pThis->lpVtbl->GetUserLangID)(pThis)

#define TIXRT_GetSessionStatus(pThis) \
	(*pThis->lpVtbl->GetSessionStatus)(pThis)

#define TIXRT_FailInstall(pThis) \
	(*pThis->lpVtbl->FailInstall)(pThis)

#define TIXRT_LogMessage \
	(*pThis->lpVtbl->LogMessage)

#define TIXRT_SetReboot(pThis) \
	(*pThis->lpVtbl->SetReboot)(pThis)

#define TIXRT_SetPostBootRegistration(pThis) \
	(*pThis->lpVtbl->SetPostBootRegistration)(pThis)

#define TIXRT_CheckFileDelete(pThis, pszFilePath) \
	(*pThis->lpVtbl->CheckFileDelete)(pThis, pszFilePath)

#define TIXRT_GetFileInstallStatus(pThis, pszFilePath) \
	(*pThis->lpVtbl->GetFileInstallStatus)(pThis, pszFilePath)

#define TIXRT_FailInstallEx(pThis, bCleanup) \
	(*pThis->lpVtbl->FailInstallEx)(pThis, bCleanup)

#define TIXRT_UpdateFeatures(pThis) \
	(*pThis->lpVtbl->UpdateFeatures)(pThis)


#endif	// __cplusplus


#ifdef __cplusplus
}		// extern "C"
#endif

#pragma pack(pop)

#endif	// __TIXRUNTIME_H
