/****************************************************************************
	$Header: /Dev/Tin3/Product/WebUpdate/TWU.h 19    31/08/06 16:01 Dave $

	Copyright (c) 1990-2005 Tarma Software Research. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	07-08-2004 14:20

	Declaration of the Tarma WebUpdate interface.

	NOTE: For best results, view with tab stops set at every 4 positions.

	THE CONTENTS OF THIS DOCUMENT ARE CONFIDENTIAL AND PROPRIETARY TO
	TARMA SOFTWARE RESEARCH PTY LTD, AND MAY NOT BE REPRODUCED, PUBLISHED,
	OR DISCLOSED TO OTHERS WITHOUT COMPANY AUTHORISATION.

	Use of this document is allowed to registered users of Tarma Installer
	and solely for the purpose of creating applications that are
	compatible with the Tarma Installer WebUpdate functionality.
****************************************************************************/

#ifndef __TARMA_TWU_H
#define __TARMA_TWU_H

#pragma once

#ifndef __TARMA_TWUDEFS_H
#include "TWUDefs.h"	// Base declarations
#endif

#ifdef __cplusplus
extern "C" {
#endif

//===========================================================================
// Overview
// --------
//
// The Tarma WebUpdate scheme allows you to perform automatic updates of
// an application from resources stored on an Internet server. The current
// version of WebUpdate supports FTP, HTTP, and HTTPS protocols.
//
// The WebUpdate scheme performs the following steps during an update:
//
//	(1)	Establish a server session with TWUOpenSession().
//	(2)	Retrieve a package information file from the Internet server
//		with TWUGetPIF().
//	(3)	For each package in the PIF:
//		(a) Compare server version with local version
//		(b) If server version is newer, download and install with
//			TWUGetPackageInstaller() and TWURunPackageInstaller()
//	(4) Close the session with TWUCloseSession().
//
// Package information file
// ------------------------
//
// The package information file is a text file in Windows .ini format
// that stores version information about the packages. During a web update
// this file is retrieved, its contents are interpreted to see if updates
// are necessary, and if they are, the actual package installers are
// retrieved and executed.
//
//	[TarmaWebUpdate]
//	Version=1
//	Packages=...list of comma-separated package names...
//
//	[...package name...]
//	Version=a.b.c.d
//	DisplayName=...display name for the package...
//	DisplayVersion=...display version for the package...
//	InstallerPath=...relative path to package installer...
//	InstallerArgs=...any command line arguments for installer...
//	DependsOn=...list of comma-separated package names...
//	ProductCode=...installed product uninstaller key...
//
// Notes:
//	- The version number format a.b.c.d is interpreted from most (a) to
//	  least (d) significant. Be careful about correctly specifying the
//	  version number; for example, 1.8.0.0 is earlier (older) than 1.60.0.0
//
//	- Many Internet servers are case-sensitive. Therefore, be sure that you
//	  specify the InstallerPath with the correct lettercase.
//
//	- Some Internet servers refuse to return a file unless it has a known
//	  and approved file name extension. Therefore, we suggest that you use
//	  .txt as the file name extension for the package information file.
//
// Package installers
// ------------------
//
// A package installer is typically a self-extracting installation
// package that installs the package after downloading. You can
// include its default command line parameters in the package information
// file, but they can be overridden or adjusted at installation time.
//
//===========================================================================

#ifdef _UNICODE
#define _TWUInstalledInfo _TWUInstalledInfoW
#define _TWUPackageData _TWUPackageDataW
#define _TWUSessionData _TWUSessionDataW
#define fTWUCallback fTWUCallbackW
#define fTWUPackageCB fTWUPackageCBW
#define TWUCheckUpdateVersion TWUCheckUpdateVersionW
#define TWUCloseSession TWUCloseSessionW
#define TWUDebugPrintf TWUDebugPrintfW
#define TWUEnumPackageDependencies TWUEnumPackageDependenciesW
#define TWUEnumPackages TWUEnumPackagesW
#define TWUFindInstalledInfo TWUFindInstalledInfoW
#define TWUFindPackage TWUFindPackageW
#define TWUGetErrorMessage TWUGetErrorMessageW
#define TWUGetEventArg TWUGetEventArgW
#define TWUGetPackage TWUGetPackageW
#define TWUGetPackageCount TWUGetPackageCountW
#define TWUGetPackageInstaller TWUGetPackageInstallerW
#define TWUGetPIF TWUGetPIFW
#define TWUGetVersionInfo TWUGetVersionInfoW
#define TWUHANDLE TWUHANDLEW
#define TWUInstalledInfo TWUInstalledInfoW
#define TWUOpenSession TWUOpenSessionW
#define TWUOpenUpdateURL TWUOpenUpdateURLW
#define TWUPackageData TWUPackageDataW
#define TWUParseVersion TWUParseVersionW
#define TWURegisterCallback TWURegisterCallbackW
#define TWURunPackageInstaller TWURunPackageInstallerW
#define TWUSessionData TWUSessionDataW
#else
#define _TWUInstalledInfo _TWUInstalledInfoA
#define _TWUPackageData _TWUPackageDataA
#define _TWUSessionData _TWUSessionDataA
#define fTWUCallback fTWUCallbackA
#define fTWUPackageCB fTWUPackageCBA
#define TWUCheckUpdateVersion TWUCheckUpdateVersionA
#define TWUCloseSession TWUCloseSessionA
#define TWUDebugPrintf TWUDebugPrintfA
#define TWUEnumPackageDependencies TWUEnumPackageDependenciesA
#define TWUEnumPackages TWUEnumPackagesA
#define TWUFindInstalledInfo TWUFindInstalledInfoA
#define TWUFindPackage TWUFindPackageA
#define TWUGetErrorMessage TWUGetErrorMessageA
#define TWUGetEventArg TWUGetEventArgA
#define TWUGetPackage TWUGetPackageA
#define TWUGetPackageCount TWUGetPackageCountA
#define TWUGetPackageInstaller TWUGetPackageInstallerA
#define TWUGetPIF TWUGetPIFA
#define TWUGetVersionInfo TWUGetVersionInfoA
#define TWUHANDLE TWUHANDLEA
#define TWUInstalledInfo TWUInstalledInfoA
#define TWUOpenSession TWUOpenSessionA
#define TWUOpenUpdateURL TWUOpenUpdateURLA
#define TWUPackageData TWUPackageDataA
#define TWUParseVersion TWUParseVersionA
#define TWURegisterCallback TWURegisterCallbackA
#define TWURunPackageInstaller TWURunPackageInstallerA
#define TWUSessionData TWUSessionDataA
#endif

//---------------------------------------------------------------------------
// Opaque TWU session handle

typedef struct _TWUSessionData *TWUHANDLE;
typedef unsigned long URESULT;	// Compatible with GetLastError(), HRESULT

//---------------------------------------------------------------------------
// Package information data

#define TWU_VERSION				1		// Current TWU format version

typedef struct _TWUPackageData
{
	UINT uStructSize;					// Must be set to sizeof(TWUPackageData)
	DWORD dwInternalUse;				// Reserved for internal use
	DWORD dwVersionLow;					// Low-order package version (c.d)
	DWORD dwVersionHigh;				// High-order package version (a.b)
	LPARAM lUserCookie1;				// Available for client use
	LPARAM lUserCookie2;				// Available for client use
	TCHAR szPackageName[64];			// Unique package name
	TCHAR szProductCode[64];			// Installed product code (uninstall key)
	TCHAR szDisplayName[256];			// Display name of the package
	TCHAR szDisplayVersion[64];			// Display version of the package
	TCHAR szInstallerPath[1024];		// Resolved path to installer resource
	TCHAR szInstallerArgs[1024];		// Command line arguments for installer
	TCHAR szDependsOn[1024];			// Dependencies
	TCHAR szDownloadPath[MAX_PATH];		// Downloaded installer path
	// From 3.3.2435 onwards:
	TCHAR szAltProductCodes[1024];		// Alternate product codes
} TWUPackageData;

#define TWU_PACKAGEDATA_SIZE_3000 \
	((3780 * sizeof(TCHAR)) + (4 * sizeof(DWORD)) + (2 * sizeof(LPARAM)))

#define TWU_PACKAGEDATA_SIZE_3003 \
	((4804 * sizeof(TCHAR)) + (4 * sizeof(DWORD)) + (2 * sizeof(LPARAM)))

// Standard names in package information file
#define TWUSTR_PIF_ALTPRODUCTCODES		_T("AltProductCodes")
#define TWUSTR_PIF_DEPENDSON			_T("DependsOn")
#define TWUSTR_PIF_DISPLAYNAME			_T("DisplayName")
#define TWUSTR_PIF_DISPLAYVERSION		_T("DisplayVersion")
#define TWUSTR_PIF_INSTALLERARGS		_T("InstallerArgs")
#define TWUSTR_PIF_INSTALLERPATH		_T("InstallerPath")
#define TWUSTR_PIF_PACKAGES				_T("Packages")
#define TWUSTR_PIF_PRODUCTCODE			_T("ProductCode")
#define TWUSTR_PIF_TWUSECTION			_T("TarmaWebUpdate")
#define TWUSTR_PIF_VERSION				_T("Version")

//---------------------------------------------------------------------------
// Callback function prototype. Callback functions must return
// ERROR_SUCCESS (0) to continue with the operation, or a nonzero error
// code to cancel the current operation.

typedef URESULT (TWUAPI *fTWUCallback)(
	TWUHANDLE hSession,					// [In] Active session handle
	int nEventCode,						// [In] Event code (see below)
	const TCHAR *pszEventMsg,			// [In] Event message text (optional)
	UINT64 ulProgress,					// [In] Progress in operation
	UINT64 ulTotal,						// [In] (Estimated) size of operation
	LPARAM lCookie						// [In] Associated client cookie
);

// Helper function to extract arguments from pszEventMsg
_TWUENTRY const TCHAR * TWUAPI TWUGetEventArg(
	const TCHAR *pszEventMsg,			// [In] String as per fTWUCallback callback
	UINT uIndex							// [In] 0-based argument index
);

// Enumeration of callback events
#define TWU_CBEVENT_SESSION_OPENING		0 
#define TWU_CBEVENT_SESSION_OPENERROR	1
#define TWU_CBEVENT_SESSION_OPENED		2
#define TWU_CBEVENT_SESSION_CLOSING		3 
#define TWU_CBEVENT_SESSION_CLOSED		4 
#define TWU_CBEVENT_CONNECTING			5 
#define TWU_CBEVENT_CONNECTERROR		6
#define TWU_CBEVENT_CONNECTED			7
#define TWU_CBEVENT_URL_OPENING			8 
#define TWU_CBEVENT_URL_OPENERROR		9 
#define TWU_CBEVENT_URL_OPENED			10
#define TWU_CBEVENT_URL_CLOSED			11
#define TWU_CBEVENT_FILE_OPENING		12
#define TWU_CBEVENT_FILE_OPENERROR		13
#define TWU_CBEVENT_FILE_OPENED			14
#define TWU_CBEVENT_FILE_CLOSED			15
#define TWU_CBEVENT_FILE_DELETING		16
#define TWU_CBEVENT_FILE_DELETED		17
#define TWU_CBEVENT_FILE_DELETEERROR	18
#define TWU_CBEVENT_DOWNLOAD_START		19
#define TWU_CBEVENT_DOWNLOAD_END		20
#define TWU_CBEVENT_DOWNLOAD_READERROR	21
#define TWU_CBEVENT_DOWNLOAD_WRITEERROR	22
#define TWU_CBEVENT_DOWNLOAD_PROGRESS	23
#define TWU_CBEVENT_PACKAGE_ENUM		24
#define TWU_CBEVENT_PROCESS_STARTING	25
#define TWU_CBEVENT_PROCESS_STARTED		26
#define TWU_CBEVENT_PROCESS_ERROR		27
#define TWU_CBEVENT_PROCESS_TERMINATED	28
#define TWU_CBEVENT_PIF_STARTPARSE		29
#define TWU_CBEVENT_PIF_ENDPARSE		30
#define TWU_CBEVENT_PIF_STARTPACKAGE	31
#define TWU_CBEVENT_PIF_ENDPACKAGE		32

//---------------------------------------------------------------------------
// Open an update session and download a package information file. If
// successful, the PIF is parsed and its contents are then available through
// TWUGetPackage(). The resulting session handle can be used to download
// and install the update packages; when done, the handle must be closed
// through a call to TWUCloseSession().
//
// This function combines TWUOpenSession() and TWUGetPIF().
//
// The passed-in PIF path should be a fully qualified URL, but may NOT
// contain port specs, user/password specs, or CGI appendages. Samples:
//
//	file://machine/share/updates/wizupdate.txt
//	ftp://ftp.acme.com/public/wizupdate.txt
//	http://www.acme.com/download/wizupdate.txt
//	https://secure.acme.com/download/wizupdate.txt
//
// Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
// on failure.

_TWUENTRY URESULT TWUAPI TWUOpenUpdateURL(
	const TCHAR *pszPIFURL,				// [In] URL of package information file
	fTWUCallback pfCallback,			// [In,opt] Callback function
	LPARAM lCallbackCookie,				// [In,opt] Associated client cookie
	TWUHANDLE *phSession				// [Out] Session handle if successful
);

//---------------------------------------------------------------------------
// Opening and closing an update session. During a single session multiple
// update checks can be performed (on the same server).
//
// Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
// on failure.

typedef enum _eTWUProtocol
{
	kTWUProtocolNone,					// Not a valid protocol
	kTWUProtocolFile,					// Local file
	kTWUProtocolFTP,					// FTP
	kTWUProtocolHTTP,					// HTTP
	kTWUProtocolHTTPS,					// HTTPS
	_kTWUProtocolMax = kTWUProtocolHTTPS
} eTWUProtocol;

#define TWU_DEFAULT_PORT		0		// Use default port # for protocol

_TWUENTRY URESULT TWUAPI TWUOpenSession(
	eTWUProtocol eProtocol,				// [In] Session protocol type
	int nPortNumber,					// [In] Port number or TWU_DEFAULT_PORT
	const TCHAR *pszServerName,			// [In] Server name
	const TCHAR *pszUserName,			// [In,opt] User name
	const TCHAR *pszPassword,			// [In,opt] Password
	fTWUCallback pfCallback,			// [In,opt] Callback function
	LPARAM lCallbackCookie,				// [In,opt] Associated client cookie
	TWUHANDLE *phSession				// [Out] Session handle if successful
);

_TWUENTRY URESULT TWUAPI TWUCloseSession(
	TWUHANDLE hSession					// [In] Session handle to close
);

//---------------------------------------------------------------------------
// Associate a callback function with the session. The callback function
// receives notifications of important events and of progress during file
// downloads.

_TWUENTRY URESULT TWUAPI TWURegisterCallback(
	TWUHANDLE hSession,					// [In] Active session handle
	fTWUCallback pfCallback,			// [In] Callback function
	LPARAM lCookie						// [In] Associated client cookie
);

//---------------------------------------------------------------------------
// Retrieve and parse a package information file from the Internet server.
// If successful, the resulting package infos can be retrieved with
// TWUGetPackage().
//
// Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
// on failure.

_TWUENTRY URESULT TWUAPI TWUGetPIF(
	TWUHANDLE hSession,					// [In] Active session handle
	const TCHAR *pszPIFPath				// [In] PIF path on server
);

//---------------------------------------------------------------------------
// Access the packages by index from 0 to TWUGetPackageCount() - 1.
//
// Returns a pointer to the package if successful, or NULL on error.

_TWUENTRY UINT TWUAPI TWUGetPackageCount(
	TWUHANDLE hSession					// [In] Active session handle
);

_TWUENTRY TWUPackageData * TWUAPI TWUGetPackage(
	TWUHANDLE hSession,					// [In] Active session handle
	UINT uIndex							// [In] Requested package index
);

_TWUENTRY TWUPackageData * TWUAPI TWUFindPackage(
	TWUHANDLE hSession,					// [In] Active session handle
	const TCHAR *pszPackageName			// [In] Requested package name
);

//---------------------------------------------------------------------------
// Enumeration callback function; must return ERROR_SUCCESS to continue,
// or a nonzero error code to terminate the enumeration and return the
// package pointer.

typedef URESULT (TWUAPI *fTWUPackageCB)(
	TWUHANDLE hSession,					// [In] Active session handle
	TWUPackageData *pPackage,			// [In] Current package
	LPARAM lCookie						// [In] Client cookie
);

//---------------------------------------------------------------------------
// Enumerate all packages.
//
// Returns NULL if all packages were enumerated, or a pointer to the package
// that caused the enumeration to be terminated.

_TWUENTRY TWUPackageData * TWUAPI TWUEnumPackages(
	TWUHANDLE hSession,					// [In] Active session handle
	fTWUPackageCB pCallback,			// [In] Callback function
	LPARAM lCookie,						// [In] Client cookie
	URESULT *puResult					// [Out,opt] Result code
);

//---------------------------------------------------------------------------
// Enumerate the package dependencies, including the package itself. This
// is recursive: dependencies of dependencies are also enumerated in a
// depth-first manner.
//
// The enumeration returns dependencies before the dependents, so it can be
// used to install packages from the bottom up. The original package is
// enumerated last.
//
// Returns NULL if all packages were enumerated, or a pointer to the package
// that caused the enumeration to be terminated.

_TWUENTRY TWUPackageData * TWUAPI TWUEnumPackageDependencies(
	TWUHANDLE hSession,					// [In] Active session handle
	TWUPackageData *pPackage,			// [In] Starting package
	fTWUPackageCB pCallback,			// [In] Callback function
	LPARAM lCookie,						// [In] Client cookie
	URESULT *puResult					// [Out,opt] Result code
);

//---------------------------------------------------------------------------
// Download a package installer. If successful, this saves the installer
// either as a temporary cached file, or as a permanent named file. In
// either case, the package data structure is updated to reflect the
// path to the downloaded installer.
//
// Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
// on failure.

_TWUENTRY URESULT TWUAPI TWUGetPackageInstaller(
	TWUHANDLE hSession,					// [In] Active session handle
	const TCHAR *pszLocalPath,			// [In,opt] Downloaded installer path
	TWUPackageData *pData				// [In/Out] Package data buffer
);

//---------------------------------------------------------------------------
// Run a downloaded package installer after TWUGetPackageInstaller().
//
// Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
// on failure.

#define TWU_RUNF_REPLACEARGS	0x00000001	// Replace instead of append pszExtraArgs
#define TWU_RUNF_SYNCHRONOUS	0x00000002	// Run installer synchronously
#define TWU_RUNF_SUSPENDED		0x00000004	// Start installer suspended
#define TWU_RUNF_SHELLEXEC		0x00000008	// Use ShellExecute()
#define TWU_RUNF_RUNAS			0x00000010	// Use RunAs (requires TWU_RUNF_SHELLEXEC)

_TWUENTRY URESULT TWUAPI TWURunPackageInstaller(
	TWUHANDLE hSession,					// [In] Active session handle
	const TWUPackageData *pData,		// [In] Package data buffer
	const TCHAR *pszExtraArgs,			// [In,opt] Additional arguments
	const TCHAR *pszDirectory,			// [In,opt] Current directory
	DWORD dwRunFlags,					// [In] Run mode flags
	PROCESS_INFORMATION *pProcessInfo,	// [Out,opt] Process information
	DWORD *pdwExitCode					// [Out,opt] Process exit code if sync
);

//---------------------------------------------------------------------------
// Delete all downloaded files.
//
// Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
// on failure.

_TWUENTRY URESULT TWUAPI TWUCleanupDownloads(
	TWUHANDLE hSession					// [In] Active session handle
);

//---------------------------------------------------------------------------
// Retrieve installation information for the currently installed version
// of a product. It requires a valid ProductCode in the package's section.
//
// The following code shows the an approach that works with software
// installed by Tarma Installer 2.x and later; you may have to adapt this
// to your own conventions if you don't use Tarma Installer (shame!).
//
// Returns ERROR_SUCCESS (0) if found, or a nonzero error code if not.

// Flags to indicate which fields are valid upon return
#define TWUII_VALID_VERSION				0x00000001	// dwVersionLow/High valid
#define TWUII_VALID_DISPLAYNAME			0x00000002	// szDisplayName valid
#define TWUII_VALID_DISPLAYVERSION		0x00000004	// szDisplayVersion valid
#define TWUII_VALID_INSTALLPATH			0x00000008	// szInstallPath valid

typedef struct _TWUInstalledInfo
{
	UINT uStructSize;					// [In] Set to sizeof(TWUInstalledInfo)
	DWORD dwValidFlags;					// [Out] Combination of TWUII_VALID_xxx
	DWORD dwVersionLow;					// [Out] Installed version LS
	DWORD dwVersionHigh;				// [Out] Installed version MS
	TCHAR szDisplayName[256];			// [Out] Display name of the product
	TCHAR szDisplayVersion[64];			// [Out] Display version of the product
	TCHAR szInstallPath[MAX_PATH];		// [Out] Main installation folder
} TWUInstalledInfo;

_TWUENTRY URESULT TWUAPI TWUFindInstalledInfo(
	TWUPackageData *pData,				// [In] Package data buffer
	TWUInstalledInfo *pInfo				// [Out] Buffer to receive info
);

//---------------------------------------------------------------------------
// Compare the file version of a local file against the version of an
// update package. If the local file does not exist or does not have a
// VERSIONINFO resource, then it always compares "older".
//
// Returns <0 if local file is older than update; 0 if equal; >0 if local
// file newer than update.

_TWUENTRY int TWUAPI TWUCheckUpdateVersion(
	const TCHAR *pszLocalPath,	// [In] Local file to check
	const TWUPackageData *pData	// [In] Update package to compare
);

//---------------------------------------------------------------------------
// Get the fixed VERSIONINFO for a file.
//
// Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
// on failure.

_TWUENTRY URESULT TWUAPI TWUGetVersionInfo(
	const TCHAR *pszFilePath,	// [In] File to retrieve version info from
	VS_FIXEDFILEINFO *pFFI		// [Out] Buffer for the version info
);

//---------------------------------------------------------------------------
// Compare two version numbers.
//
// Returns <0 if first is older than second; 0 if equal; >0 if first newer
// than second.

_TWUENTRY int TWUAPI TWUCompareVersions(
	DWORD dwVersionLow1,		// [In] Low-order DWORD of first version
	DWORD dwVersionHigh1,		// [In] High-order DWORD of first version
	DWORD dwVersionLow2,		// [In] Low-order DWORD of second version
	DWORD dwVersionHigh2		// [In] High-order DWORD of second version
);

//---------------------------------------------------------------------------
// Parse a string of the form a.b.c.d to a version-compatible number.
// The a.b part is returned as (65536 * a) + b in the high-order DWORD;
// the c.d is returned as (65536 * c) + d in the low-order DWORD. This is
// compatible with the usual VERSIONINFO conventions.

_TWUENTRY BOOL TWUAPI TWUParseVersion(
	const TCHAR *pszVersion,	// [In] Version string as a.b.c.d
	DWORD *pdwVersionLow,		// [Out] Pointer to low-order version DWORD
	DWORD *pdwVersionHigh		// [Out] Pointer to high-order version DWORD
);

//---------------------------------------------------------------------------
// Retrieve a Win32 error message text.

_TWUENTRY void TWUAPI TWUGetErrorMessage(
	URESULT uResult,			// [In] Win32 error code to look up
	TCHAR *pszBuffer,			// [Out] Output buffer for message
	UINT uBufferSize			// [In] Size of buffer in TCHARs (>= 32)
);

#ifdef _DEBUG
//---------------------------------------------------------------------------
// Write a formatted string to the debugging log

_TWUENTRY void TWUAPI TWUDebugPrintf(
	const TCHAR *pszFormat,		// [In] Formatting string
	...							// [In] Formatting arguments
);

// Use the following macro with *double* parentheses, like this:
//
//	TWULOG((_T("%s: failed with error %u\n"), pszPath, uResult));

#define TWULOG(x)		TWUDebugPrintf x

#else	// Not _DEBUG

#define TWULOG(x)		((void)0)

#endif	// _DEBUG

#ifdef __cplusplus
}		// extern "C"
#endif

#endif	// __TARMA_TWU_H
