{****************************************************************************
	$Header: /Dev/Tin3/Product/WebUpdate/TWU.pas 2     10-04-06 19:39 Ron $

	Copyright (c) 1990-2006 Tarma Software Research. All rights reserved.

	Project:	Tarma Installer
	Author:		Ron van der Wal
	Created:	10-04-2006 14:27

	Declaration of the Tarma WebUpdate interface, Delphi version.

	THE CONTENTS OF THIS DOCUMENT ARE CONFIDENTIAL AND PROPRIETARY TO
	TARMA SOFTWARE RESEARCH PTY LTD, AND MAY NOT BE REPRODUCED, PUBLISHED,
	OR DISCLOSED TO OTHERS WITHOUT COMPANY AUTHORISATION.

	Use of this document is allowed to registered users of Tarma Installer
	and solely for the purpose of creating applications that are
	compatible with the Tarma Installer WebUpdate functionality.
****************************************************************************}

unit TWU;

interface

uses Windows;

{---------------------------------------------------------------------------}
{ Standard types }

type
	TWUHANDLE = Pointer;
	PTWUHANDLE = ^TWUHANDLE;
	URESULT = ULONG;
	LPARAM = Longint;

{$IFDEF UNICODE}

type
	TCHAR = WideChar;
	PTCHAR = PWideChar;

{$ELSE}

type
	TCHAR = AnsiChar;
	PTCHAR = PAnsiChar;

{$ENDIF}

{---------------------------------------------------------------------------}
{ Package information data }

const TWU_VERSION = 1;			{ Current TWU format version }

type TWUPackageData = record
	uStructSize: UINT;			{ Must be set to sizeof(TWUPackageData) }
	dwInternalUse: DWORD;		{ Reserved for internal use }
	dwVersionLow: DWORD;		{ Low-order package version (c.d) }
	dwVersionHigh: DWORD;		{ High-order package version (a.b) }
	lUserCookie1: LPARAM;		{ Available for client use }
	lUserCookie2: LPARAM;		{ Available for client use }
	szPackageName: array [0..63] of TCHAR;		{ Unique package name }
	szProductCode: array [0..63] of TCHAR;		{ Installed product code (uninstall key) }
	szDisplayName: array [0..255] of TCHAR;		{ Display name of the package }
	szDisplayVersion: array [0..63] of TCHAR;	{ Display version of the package }
	szInstallerPath: array [0..1023] of TCHAR;	{ Resolved path to installer resource }
	szInstallerArgs: array [0..1023] of TCHAR;	{ Command line arguments for installer }
	szDependsOn: array [0..1023] of TCHAR;		{ Dependencies }
	szDownloadPath: array [0..259] of TCHAR;	{ Downloaded installer path }
end;

type
	PTWUPackageData = ^TWUPackageData;

{ Standard names in package information file }
const
	TWUSTR_PIF_DEPENDSON = 'DependsOn';
	TWUSTR_PIF_DISPLAYNAME = 'DisplayName';
	TWUSTR_PIF_DISPLAYVERSION = 'DisplayVersion';
	TWUSTR_PIF_INSTALLERARGS = 'InstallerArgs';
	TWUSTR_PIF_INSTALLERPATH = 'InstallerPath';
	TWUSTR_PIF_PACKAGES = 'Packages';
	TWUSTR_PIF_PRODUCTCODE = 'ProductCode';
	TWUSTR_PIF_TWUSECTION = 'TarmaWebUpdate';
	TWUSTR_PIF_VERSION = 'Version';

{---------------------------------------------------------------------------
 Open an update session and download a package information file. If
 successful, the PIF is parsed and its contents are then available through
 TWUGetPackage(). The resulting session handle can be used to download
 and install the update packages; when done, the handle must be closed
 through a call to TWUCloseSession().

 This function combines TWUOpenSession() and TWUGetPIF().

 The passed-in PIF path should be a fully qualified URL, but may NOT
 contain port specs, user/password specs, or CGI appendages. Samples:

	file:{machine/share/updates/wizupdate.txt
	ftp:{ftp.acme.com/public/wizupdate.txt
	http:{www.acme.com/download/wizupdate.txt
	https:{secure.acme.com/download/wizupdate.txt

 Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
 on failure.
}

function TWUOpenUpdateURL(
	pszPIFURL: PTCHAR;			{ [In] URL of package information file }
	var phSession: TWUHANDLE	{ [Out] Session handle if successful }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUOpenUpdateURLW@8';
{$ELSE}
external 'TWU32.dll' name '_TWUOpenUpdateURLA@8';
{$ENDIF}

{---------------------------------------------------------------------------
 Opening and closing an update session. During a single session multiple
 update checks can be performed (on the same server).

 Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
 on failure.
}

type eTWUProtocol =
(
	kTWUProtocolNone,			{ Not a valid protocol }
	kTWUProtocolFile,			{ Local file }
	kTWUProtocolFTP,			{ FTP }
	kTWUProtocolHTTP,			{ HTTP }
	kTWUProtocolHTTPS			{ HTTPS }
);

const TWU_DEFAULT_PORT = 0;	{ Use default port # for protocol }

function TWUOpenSession(
	eProtocol: eTWUProtocol;	{ [In] Session protocol type }
	nPortNumber: Integer;		{ [In] Port number or TWU_DEFAULT_PORT }
	pszServerName: PTCHAR;		{ [In] Server name }
	pszUserName: PTCHAR;		{ [In,opt] User name }
	pszPassword: PTCHAR;		{ [In,opt] Password }
	var phSession: TWUHANDLE	{ [Out] Session handle if successful }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUOpenSessionW@24';
{$ELSE}
external 'TWU32.dll' name '_TWUOpenSessionA@24';
{$ENDIF}

procedure TWUCloseSession(
	hSession: TWUHANDLE 		{ [In] Session handle to close }
); stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUCloseSessionW@4';
{$ELSE}
external 'TWU32.dll' name '_TWUCloseSessionA@4';
{$ENDIF}

{---------------------------------------------------------------------------
 Retrieve and parse a package information file from the Internet server.
 If successful, the resulting package infos can be retrieved with
 TWUGetPackage().

 Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
 on failure.
}

function TWUGetPIF(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pszPIFPath: PTCHAR			{ [In] PIF path on server }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUGetPIFW@8';
{$ELSE}
external 'TWU32.dll' name '_TWUGetPIFA@8';
{$ENDIF}

{---------------------------------------------------------------------------
{ Access the packages by index from 0 to TWUGetPackageCount() - 1.

 Returns a pointer to the package if successful, or NULL on error.
}

function TWUGetPackageCount(
	hSession: TWUHANDLE			{ [In] Active session handle }
): UINT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUGetPackageCount@4';
{$ELSE}
external 'TWU32.dll' name '_TWUGetPackageCount@4';
{$ENDIF}

function TWUGetPackage(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	uIndex: UINT				{ [In] Requested package index }
): PTWUPackageData; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUGetPackageW@8';
{$ELSE}
external 'TWU32.dll' name '_TWUGetPackageA@8';
{$ENDIF}

function TWUFindPackage(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pszPackageName: PTCHAR		{ [In] Requested package name }
): PTWUPackageData; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUFindPackageW@8';
{$ELSE}
external 'TWU32.dll' name '_TWUFindPackageA@8';
{$ENDIF}

{---------------------------------------------------------------------------
 Enumeration callback function; must return ERROR_SUCCESS to continue,
 or a nonzero error code to terminate the enumeration and return the
 package pointer.
}

type
	fTWUPackageCB = function(
		hSession: TWUHANDLE;	{ [In] Active session handle }
		pPackage: PTWUPackageData;	{ [In] Current package }
		lCookie: LPARAM		    { [In] Client cookie }
	): URESULT; stdcall;

{---------------------------------------------------------------------------
 Enumerate all packages.

 Returns NULL if all packages were enumerated, or a pointer to the package
 that caused the enumeration to be terminated.
}

function TWUEnumPackages(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pCallback: fTWUPackageCB;	{ [In] Callback function }
	lCookie: LPARAM;			{ [In] Client cookie }
	var puResult: URESULT		{ [Out] Result code }
): PTWUPackageData; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUEnumPackagesW@16';
{$ELSE}
external 'TWU32.dll' name '_TWUEnumPackagesA@16';
{$ENDIF}

{---------------------------------------------------------------------------
 Enumerate the package dependencies, including the package itself. This
 is recursive: dependencies of dependencies are also enumerated in a
 depth-first manner.

 The enumeration returns dependencies before the dependents, so it can be
 used to install packages from the bottom up. The original package is
 enumerated last.

 Returns NULL if all packages were enumerated, or a pointer to the package
 that caused the enumeration to be terminated.
}

function TWUEnumPackageDependencies(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pPackage: PTWUPackageData;	{ [In] Starting package }
	pCallback: fTWUPackageCB;	{ [In] Callback function }
	lCookie: LPARAM;		    { [In] Client cookie }
	var puResult: URESULT		{ [Out] Result code }
): PTWUPackageData; stdcall

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUEnumPackageDependenciesW@20';
{$ELSE}
external 'TWU32.dll' name '_TWUEnumPackageDependenciesA@20';
{$ENDIF}

{---------------------------------------------------------------------------
 Download a package installer. If successful, this saves the installer
 either as a temporary cached file, or as a permanent named file. In
 either case, the package data structure is updated to reflect the
 path to the downloaded installer.

 Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
 on failure.
}

function TWUGetPackageInstaller(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pszLocalPath: PTCHAR;		{ [In,opt] Downloaded installer path }
	var pData: TWUPackageData	{ [In/Out] Package data buffer }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUGetPackageInstallerW@12';
{$ELSE}
external 'TWU32.dll' name '_TWUGetPackageInstallerA@12';
{$ENDIF}

{---------------------------------------------------------------------------
 Run a downloaded package installer after TWUGetPackageInstaller().

 Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
 on failure.
}

const
  TWU_RUNF_REPLACEARGS =	$00000001;	{ Replace instead of append pszExtraArgs }
  TWU_RUNF_SYNCHRONOUS =	$00000002;	{ Run installer synchronously }
  TWU_RUNF_SUSPENDED =		$00000004;	{ Start installer suspended }
  TWU_RUNF_SHELLEXEC =		$00000008;	{ Use ShellExecute() }
  TWU_RUNF_RUNAS =			  $00000010;	{ Use RunAs (requires TWU_RUNF_SHELLEXEC) }

function TWURunPackageInstaller(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pData: PTWUPackageData;		{ [In] Package data buffer }
	pszExtraArgs: PTCHAR;		{ [In,opt] Additional arguments }
	pszDirectory: PTCHAR;		{ [In,opt] Current directory }
	dwRunFlags: DWORD;			{ [In] Run mode flags }
	var pProcessInfo: PROCESS_INFORMATION;{ [Out] Process information }
	var pdwExitCode: DWORD		{ [Out] Process exit code if sync }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWURunPackageInstallerW@28';
{$ELSE}
external 'TWU32.dll' name '_TWURunPackageInstallerA@28';
{$ENDIF}

{---------------------------------------------------------------------------
 Delete all downloaded files.

 Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
 on failure.
}

function TWUCleanupDownloads(
	hSession: TWUHANDLE			{ [In] Active session handle }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUCleanupDownloads@4';
{$ELSE}
external 'TWU32.dll' name '_TWUCleanupDownloads@4';
{$ENDIF}

{---------------------------------------------------------------------------
 Retrieve installation information for the currently installed version
 of a product. It requires a valid ProductCode in the package's section.

 The following code shows the an approach that works with software
 installed by Tarma Installer 2.x and later; you may have to adapt this
 to your own conventions if you don't use Tarma Installer (shame!).

 Returns ERROR_SUCCESS (0) if found, or a nonzero error code if not.
}

{ Flags to indicate which fields are valid upon return }
const
  TWUII_VALID_VERSION =			$00000001;	{ dwVersionLow/High valid }
  TWUII_VALID_DISPLAYNAME =		$00000002;	{ szDisplayName valid }
  TWUII_VALID_DISPLAYVERSION =	$00000004;	{ szDisplayVersion valid }
  TWUII_VALID_INSTALLPATH =		$00000008;	{ szInstallPath valid }

type TWUInstalledInfo = record
	uStructSize: UINT;			{ [In] Set to sizeof(TWUInstalledInfo) }
	dwValidFlags: DWORD;		{ [Out] Combination of TWUII_VALID_xxx }
	dwVersionLow: DWORD;		{ [Out] Installed version LS }
	dwVersionHigh: DWORD;		{ [Out] Installed version MS }
	szDisplayName: array[0..255] of TCHAR;		{ [Out] Display name of the product }
	szDisplayVersion: array[0..63] of TCHAR;	{ [Out] Display version of the product }
	szInstallPath: array[0..259] of TCHAR;		{ [Out] Main installation folder }
end;

type
	PTWUInstalledInfo = ^TWUInstalledInfo;

function TWUFindInstalledInfo(
	pData: PTWUPackageData;		{ [In] Package data buffer }
	var pInfo: TWUInstalledInfo { [Out] Buffer to receive info }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUFindInstalledInfoW@8';
{$ELSE}
external 'TWU32.dll' name '_TWUFindInstalledInfoA@8';
{$ENDIF}

{---------------------------------------------------------------------------
 Compare the file version of a local file against the version of an
 update package. If the local file does not exist or does not have a
 VERSIONINFO resource, then it always compares "older".

 Returns <0 if local file is older than update; 0 if equal; >0 if local
 file newer than update.
}

function TWUCheckUpdateVersion(
	pszLocalPath: PTCHAR;		{ [In] Local file to check }
	pData: PTWUPackageData		{ [In] Update package to compare }
): Integer; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUCheckUpdateVersionW@8';
{$ELSE}
external 'TWU32.dll' name '_TWUCheckUpdateVersionA@8';
{$ENDIF}

{---------------------------------------------------------------------------
 Get the fixed VERSIONINFO for a file.

 Returns ERROR_SUCCESS (0) if successful, or a nonzero Win32 error code
 on failure.
}

function TWUGetVersionInfo(
	pszFilePath: PTCHAR;		{ [In] File to retrieve version info from }
	var pFFI: VS_FIXEDFILEINFO	{ [Out] Buffer for the version info }
): URESULT; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUGetVersionInfoW@8';
{$ELSE}
external 'TWU32.dll' name '_TWUGetVersionInfoA@8';
{$ENDIF}

{---------------------------------------------------------------------------
 Compare two version numbers.

 Returns <0 if first is older than second; 0 if equal; >0 if first newer
 than second.
}

function TWUCompareVersions(
	dwVersionLow1: DWORD;		{ [In] Low-order DWORD of first version }
	dwVersionHigh1: DWORD;		{ [In] High-order DWORD of first version }
	dwVersionLow2: DWORD;		{ [In] Low-order DWORD of second version }
	dwVersionHigh2: DWORD		{ [In] High-order DWORD of second version }
): Integer; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUCompareVersions@16';
{$ELSE}
external 'TWU32.dll' name '_TWUCompareVersions@16';
{$ENDIF}

{---------------------------------------------------------------------------
 Parse a string of the form a.b.c.d to a version-compatible number.
 The a.b part is returned as (65536 * a) + b in the high-order DWORD;
 the c.d is returned as (65536 * c) + d in the low-order DWORD. This is
 compatible with the usual VERSIONINFO conventions.
}

function TWUParseVersion(
	pszVersion: PTCHAR;			{ [In] Version string as a.b.c.d }
	var pdwVersionLow: DWORD;	{ [Out] Pointer to low-order version DWORD }
	var pdwVersionHigh: DWORD	{ [Out] Pointer to high-order version DWORD }
): BOOL; stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUParseVersionW@12';
{$ELSE}
external 'TWU32.dll' name '_TWUParseVersionA@12';
{$ENDIF}

{---------------------------------------------------------------------------
 Retrieve a Win32 error message text.
}

procedure TWUGetErrorMessage(
	_uResult: URESULT;			{ [In] Win32 error code to look up }
	pszBuffer: PTCHAR;			{ [Out] Output buffer for message }
	uBufferSize: UINT			{ [In] Size of buffer in TCHARs (>= 32) }
); stdcall;

{$IFDEF UNICODE}
external 'TWUNT.dll' name '_TWUGetErrorMessageW@12';
{$ELSE}
external 'TWU32.dll' name '_TWUGetErrorMessageA@12';
{$ENDIF}

implementation

end.
