program WebUpdate;

{$APPTYPE CONSOLE}

uses
	SysUtils, TWU, Windows;

{---------------------------------------------------------------------------
	Global variables
}

var
	g_bInstall: BOOL;
	g_bUpdate: BOOL;

{---------------------------------------------------------------------------
	Print usage summary
}

procedure PrintUsage(pszArg: PTCHAR);
begin
	if (pszArg <> nil) and (pszArg^ <> #0) then
		writeln('Unknown option ', pszArg^);

	writeln('Usage: WebUpdate [/install][/update] package pifURL');
	writeln;
	writeln('WebUpdate retrieves the package info from the URL and installs or updates');
	writeln('the package "package". If neither /install nor /update is given, packages');
	writeln('are merely listed without being downloaded or installed. Options:');
	writeln;
	writeln('  /install  - install the packages always');
	writeln('  /update   - install the packages only if present and outdated');
	writeln;
	writeln('You can test this sample by specifying the following command:');
	writeln;
	writeln('  webupdate /install tin3 http://www.tarma.com/download/update.txt');
end;

{---------------------------------------------------------------------------
	Enumeration callback that downloads required packages. It sets
	pPackage^.lUserCookie1 to nonzero if successful.
}

function _WUDownloadCB(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pPackage: PTWUPackageData;	{ [In] Current package }
	lCookie: LPARAM				{ [In] Client cookie }
): URESULT; stdcall;
var
	_uResult: URESULT;
	bDownload: BOOL;
	ii: TWUInstalledInfo;
begin
	_uResult := ERROR_SUCCESS;	{ To continue enumeration }
	bDownload := FALSE;

	writeln('Package ', pPackage^.szPackageName, ': Available version=',
		HIWORD(pPackage^.dwVersionHigh), '.',
		LOWORD(pPackage^.dwVersionHigh), '.',
		HIWORD(pPackage^.dwVersionLow), '.',
		LOWORD(pPackage^.dwVersionLow));

	{ Find out what to do, depending on command line options }
	if g_bInstall then
	begin
		bDownload := TRUE;		{ Always download the package }
	end
	else if g_bUpdate then
	begin
		{ Get information about the currently installed version }
		ii.uStructSize := sizeof(TWUInstalledInfo);
		_uResult := TWUFindInstalledInfo(pPackage, ii);
		if _uResult = ERROR_SUCCESS then
		begin
			writeln('Package ', pPackage^.szPackageName, ': Installed version=',
				HIWORD(ii.dwVersionHigh), '.',
				LOWORD(ii.dwVersionHigh), '.',
				HIWORD(ii.dwVersionLow), '.',
				LOWORD(ii.dwVersionLow));

			bDownload := TWUCompareVersions(ii.dwVersionLow, ii.dwVersionHigh,
				pPackage^.dwVersionLow, pPackage^.dwVersionHigh) < 0;
		end;
		_uResult := ERROR_SUCCESS;
	end;

	{ Download the package installer if required and mark the package if successful. }
	if bDownload then
	begin
		writeln('Downloading package ', pPackage^.szPackageName,
			': ', pPackage^.szInstallerPath, '...');

		_uResult := TWUGetPackageInstaller(hSession, nil, pPackage^);
		if _uResult = ERROR_SUCCESS then pPackage^.lUserCookie1 := 1;

		writeln('Downloaded package ', pPackage^.szPackageName,
			': ', pPackage^.szDownloadPath, '; result=', _uResult);
	end
	else
	begin
		writeln('Package ', pPackage^.szPackageName,
			' does not need to be downloaded');
	end;
	_WUDownloadCB := _uResult;
end;

{---------------------------------------------------------------------------
	Enumeration callback that installs required packages
}

function _WUInstallCB(
	hSession: TWUHANDLE;		{ [In] Active session handle }
	pPackage: PTWUPackageData;	{ [In] Current package }
	lCookie: LPARAM				{ [In] Client cookie }
): URESULT; stdcall;
var
	_uResult: URESULT;
	dwExitCode: DWORD;
	pi: PROCESS_INFORMATION;
begin
	_uResult := ERROR_SUCCESS;	{ To continue enumeration }

	{ Install the package if it was successfully downloaded }
	if pPackage^.lUserCookie1 <> 0 then
	begin
		writeln('Installing package ', pPackage^.szPackageName,
			': ', pPackage^.szDisplayName, '...');

		_uResult := TWURunPackageInstaller(hSession, pPackage, nil, nil,
			TWU_RUNF_SYNCHRONOUS, pi, dwExitCode);

		writeln('Installed package ', pPackage^.szPackageName,
			': ', pPackage^.szDownloadPath, '; result=', _uResult,
			', exit code=', dwExitCode);
	end;
	_WUInstallCB := _uResult;
end;

var
	szPackageName: array[0..63] of TCHAR;
	szURL: array[0..1023] of TCHAR;
	szMessage: array[0..255] of TCHAR;
	hSession: TWUHANDLE;
	pPackage, pFailedPackage: PTWUPackageData;
	_uResult: URESULT;
	nResult, i: Integer;

label
	lblExit;

begin
	{ TODO -oUser -cConsole Main : Insert code here }

	nResult := 0;
	szPackageName[0] := #0;
	szURL[0] := #0;

	{ Parse the command line arguments }
	for i := 1 to ParamCount do
	begin
		if ParamStr(i)[1] = '/' then
		begin
			{ Option }
			if ParamStr(i) = '/install' then
			begin
				g_bInstall := TRUE;
				g_bUpdate := FALSE;
			end
			else if ParamStr(i) = '/update' then
			begin
				g_bInstall := FALSE;
				g_bUpdate := TRUE;
			end
			else
			begin
				PrintUsage(@ParamStr(i)[1]);
				Halt(Word(-1));
			end
		end
		else
		begin
			{ Package name or URL }
			if szPackageName[0] = #0 then
			begin
				StrLCopy(szPackageName, PTCHAR(ParamStr(i)), Length(szPackageName));
			end
			else if szURL[0] = #0 then
			begin
				StrLCopy(szURL, PTCHAR(ParamStr(i)), Length(szURL));
			end
			else
			begin
				PrintUsage(PTCHAR(ParamStr(i)));
				Halt(Word(-1));
			end;
		end;
	end;

	{ Did we get a package name and URL? }
	if (szPackageName[0] = #0) or (szURL[0] = #0) then
	begin
		PrintUsage(nil);
		Halt(Word(-1));
	end;

	{ Retrieve and parse the package information file }
	writeln('Retrieving package information from ', szURL, '...');
	_uResult := TWUOpenUpdateURL(@szURL, hSession);
	if _uResult <> ERROR_SUCCESS then
	begin
		TWUGetErrorMessage(_uResult, @szMessage, Length(szMessage));
		writeln('Error ', _uResult, ' while opening URL ', szURL,
			': ', szMessage);
		Halt(Word(-1));
	end;

	{ Find the desired package }
	pPackage := TWUFindPackage(hSession, @szPackageName);
	if pPackage = nil then
	begin
		writeln('Package ', szPackageName, ' not found at URL ', szURL);
		nResult := 2;
		goto lblExit;
	end;

	{ First pass: Download any packages that must be downloaded }
	pFailedPackage := TWUEnumPackageDependencies(hSession, pPackage,
		_WUDownloadCB, 0, _uResult);
	if (pFailedPackage <> nil) or (_uResult <> ERROR_SUCCESS) then
	begin
		TWUGetErrorMessage(_uResult, @szMessage, Length(szMessage));
		writeln('Error ', _uResult, ' while downloading package ',
			szPackageName, ': ', szMessage);
		nResult := 3;
		goto lblExit;
	end;

	{ Second pass: Install any packages that must be installed }
	pFailedPackage := TWUEnumPackageDependencies(hSession, pPackage,
		_WUInstallCB, 0, _uResult);
	if (pFailedPackage <> nil) or (_uResult <> ERROR_SUCCESS) then
	begin
		TWUGetErrorMessage(_uResult, @szMessage, Length(szMessage));
		writeln('Error ', _uResult, ' while installing package ',
			szPackageName, ': ', szMessage);
		nResult := 4;
		goto lblExit;
	end;

lblExit:
	{ Clean up }
	TWUCleanupDownloads(hSession);
	TWUCloseSession(hSession);
	Halt(nResult);
end.
